@@ -3,7 +3,7 @@
 /**
  * Assignment to array makes it unsafe to return it as original generic type
  */
-function f<Tv as KeyedContainer<mixed, mixed>>(Tv $a): Tv {
+function f<Tv as array<mixed, mixed>>(Tv $a): Tv {
   $a['a'] = 4;
   return $a;
 }
