@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2015, Facebook, Inc.
+ *  Copyright (c) 2017, Facebook, Inc.
  *  All rights reserved.
  *
  *  This source code is licensed under the BSD-style license found in the
@@ -13,10 +13,11 @@
 
 #include <folly/Range.h>
 
-#include "mcrouter/lib/mc/msg.h"
 #include "mcrouter/TkoCounters.h"
+#include "mcrouter/lib/mc/msg.h"
 
-namespace facebook { namespace memcache {
+namespace facebook {
+namespace memcache {
 
 struct AccessPoint;
 
@@ -45,5 +46,6 @@ struct TkoLog {
   const TkoCounters& globalTkos;
   folly::StringPiece poolName;
 };
-
-}}}  // facebook::memcache::mcrouter
+}
+}
+} // facebook::memcache::mcrouter
